#!/usr/bin/env python

from __future__ import print_function
import argparse
import sys
import os.path
from numpy import random
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO


__version__ = '0.2.0'


def read_fasta_sequence(fasta_file_path):
    """
    Read a fasta file and return the sequence as a string.

    Parameters
    ----------
    fasta_file_path : str
        Path to the fasta file to read.

    Returns
    -------
    seq_name : str
        The fasta description of the first sequence in the file only.
    sequence : str
        Sequence string.
        Note: if the fast file contains multiple sequences, all the sequences
        are combined into a single string.
    """
    seq_string = []
    with open(fasta_file_path, "r") as infile:
        for idx, line in enumerate(infile):
            if idx == 0:
                seq_name = line.strip(">").strip("\n")
            elif ">" in line:
                pass
            else:
                seq_string.append(line.strip("\n"))

    seq_str = "".join(seq_string)
    return (seq_name, seq_str)


def write_fasta_sequence(seq_id, file_path, sequence_list, mutations):
    """
    Write a mutated sequence to a fasta file.  The fasta defline is suffixed
    with a mutation description of the form (mutated s=100 i=20 d=20)
    describing the number of substitutions, insertions, and deletions.

    Parameters
    ----------
    seq_id : str
        ID of the sequence which will be written to the fasta description line.
    file_path : str
        Path to the fasta file to write.  An existing file will be overwritten.
    sequence_list : list of str
        List of sequence fragments which may be 0 or more bases.  The fragments
        are concatenated before writing to the fast a file.
    mutations : tuple
        3-tuple of number of substitutions, insertions, and deletions in that
        order.  The mutation counts are appended to the fasta defline.
    """
    seq_str = "".join(sequence_list)
    seq = Seq(seq_str)
    description = "(mutated s=%i i=%i d=%i)" % mutations
    record = SeqRecord(seq, id=seq_id, description=description)
    SeqIO.write([record], file_path, "fasta")


def get_eligible_positions(seq_str):
    """
    Find all the positions where the original base is ACTG.  Those are the only
    positions eligible for mutation.

    Parameters
    ----------
    seq_str : str
        Sequence string.

    Returns
    -------
    eligible_positions : list of integers
        List of all positions where the base is eligible for mutation.
    """
    eligible_positions = list()
    pos = 0
    for base in seq_str:
        if base.upper() in "ACGT":
            eligible_positions.append(pos)
        pos += 1
    return eligible_positions


def build_mutated_seq(seq_str, eligible_positions, num_subs, num_insertions, num_deletions):
    """
    Copy a sequence and randomly introduce the specified numbers of
    substitutions, insertions, and deletions.

    Parameters
    ----------
    seq_str : str
        Sequence string.
    eligible_positions : list of integers
        Positions where the original base is eligible for mutating
    num_subs : int
        Number of base substitutions to make.
    num_insertions : int
        Number of base insertions to make.  Insertions are placed before the
        original base at positions having insertions.
    num_deletions : int
        Number of base deletions to make.

    Returns
    -------
    new_indexed_seq : list of str
        List indexed by original position containing strings of bases.  Each
        string can be 0 - 2 bases long, where a zero length string indicates
        a deletion at the position, a string containing a single base indicates
        no mutation at the position, and a string containing two bases
        indicates an insertion prior to the original base at the position.
    subs_positions : array of int
        Array of positions where substitions are introduced.
    insertion_positions : array of int
        Array of positions where insertions are introduced.
    deletion_positions : array of int
        Array of positions where bases are deleted.
    """
    substitution_choices = {"A" : ["C", "T", "G"],
                            "C" : ["A", "T", "G"],
                            "T" : ["C", "A", "G"],
                            "G" : ["C", "T", "A"],
                           }

    seq_length = len(seq_str)
    num_mutations = num_subs + num_deletions + num_insertions
    positions = random.choice(eligible_positions, size=num_mutations, replace=False)
    subs_positions = positions[: num_subs]
    deletion_positions = positions[num_subs : (num_subs + num_deletions)]
    insertion_positions = positions[(num_subs + num_deletions) : len(positions)]

    # Copy the original sequence in a way that easily allows mutations
    # while preserving position information
    new_indexed_seq = list(seq_str)

    for i in subs_positions:
        original_base = seq_str[i]
        upper_original_base = original_base.upper()
        if upper_original_base in substitution_choices:
            new_base = random.choice(substitution_choices[upper_original_base], size=1)[0]
            new_indexed_seq[i] = new_base
        else:
            print('Warning: unexpected base "%s" at position %i.  There will be no mutation at this position.' % (original_base, i), file=sys.stderr)

    for i in deletion_positions:
        new_indexed_seq[i] = ""

    for i in insertion_positions:
        original_base = seq_str[i]
        insert_base = random.choice(["A", "C", "T", "G"], size=1)[0]
        new_indexed_seq[i] = insert_base + original_base

    return (new_indexed_seq, subs_positions, insertion_positions, deletion_positions, )


def run_simulations(seq_str, eligible_positions, base_file_name, seq_name, num_sims, num_subs, num_insertions, num_deletions, summary_file_path=None):
    """
    Generate multiple random mutations of a reference sequence, repeatedly 
    calling build_mutated_seq() to create each of the mutated sequences.

    Parameters
    ----------
    seq_str : str
        Original sequence string.
    eligible_positions : list of integers
        Positions where the original base is eligible for mutating
    base_file_name : str
        The base file name of the original reference with the extension
        removed.  This will be the file name prefix of the generated mutated
        files.
    seq_name : str
        ID of the sequence which will be written to the fasta description line.
    num_sims : int
        Number of mutated sequenced to generate.
    num_subs : int
        Number of base substitutions to make.
    num_insertions : int
        Number of base insertions to make.  Insertions are placed before the
        original base at positions having insertions.
    num_deletions : int
        Number of base deletions to make.
    summary_file_path : str, optional
        Path to summary file where a list of positions and corresponding
        mutations will be written.
    """
    if summary_file_path:
        snp_list_file = open(summary_file_path, "w")

    try:
        if summary_file_path:
            snp_list_file.write("Replicate\tPosition\tOriginalBase\tNewBase\n")

        for replicate in range(1, num_sims + 1):
            print("Creating replicate %i" % replicate)
            replicate_name = base_file_name + "_mutated_" + str(replicate)

            new_indexed_seq, subs_positions, insertion_positions, deletion_positions = \
                build_mutated_seq(seq_str, eligible_positions, num_subs, num_insertions, num_deletions)

            mutations = (num_subs, num_insertions, num_deletions)
            write_fasta_sequence(seq_name, replicate_name + ".fasta", new_indexed_seq, mutations)

            if summary_file_path:
                summary_list = list()
                summary_list.extend([(pos, new_indexed_seq[pos]) for pos in subs_positions])
                summary_list.extend([(pos, new_indexed_seq[pos] + "_insertion") for pos in insertion_positions])
                summary_list.extend([(pos, "_deletion") for pos in deletion_positions])
                for pos, change in sorted(summary_list):
                    snp_list_file.write("%s\t%i\t%s\t%s\n" % (replicate_name, pos + 1, seq_str[pos], change))

    finally:
        if summary_file_path:
            snp_list_file.close()


def parse_arguments(system_args):
    """
    Parse command line arguments.

    Parameters
    ----------
    system_args : list
        List of command line arguments, usually sys.argv[1:].

    Returns
    -------
    Namespace
        Command line arguments are stored as attributes of a Namespace.
    """
    usage = """Generate mutated sequence files from a reference genome.  Takes a fasta file and creates
               a specified number of randomly generated base substitutions, insertions, and deletions.
               Outputs the mutated genomes, and optionally, a summary file listing the mutations by
               position."""
               
    def non_negative_int(value):
        try:
            ivalue = int(value)
        except:
            raise argparse.ArgumentTypeError("Must be a number >= 0")
        if ivalue < 0:
            raise argparse.ArgumentTypeError("Must be >= 0")
        return ivalue
               
    def positive_int(value):
        try:
            ivalue = int(value)
        except:
            raise argparse.ArgumentTypeError("Must be a number greater than 0")
        if ivalue <= 0:
            raise argparse.ArgumentTypeError("Must be greater than 0")
        return ivalue

    parser = argparse.ArgumentParser(description=usage, formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument(                                             dest="input_fasta_file", type=str,                            help="Input fasta file.")
    parser.add_argument("-o", "--summary",           metavar="FILE", dest="summary_file",     type=str,              default=None, help="Output positional summary file.")
    parser.add_argument("-n", "--num-simulations",   metavar="INT",  dest="num_sims",         type=positive_int,     default=100,  help="Number of mutated sequences to generate.")
    parser.add_argument("-s", "--num-substitutions", metavar="INT",  dest="num_subs",         type=non_negative_int, default=500,  help="Number of substitutions.")
    parser.add_argument("-i", "--num-insertions",    metavar="INT",  dest="num_insertions",   type=non_negative_int, default=20,   help="Number of insertions.")
    parser.add_argument("-d", "--num-deletions",     metavar="INT",  dest="num_deletions",    type=non_negative_int, default=20,   help="Number of deletions.")
    parser.add_argument("-r", "--random-seed",       metavar="INT",  dest="random_seed",      type=int,              default=None, help="Random number seed; if not set, the results are not reproducible.")
    parser.add_argument('--version', action='version', version='%(prog)s version ' + __version__)

    args = parser.parse_args(system_args)
    return args


def main(args):
    """
    Generate multiple random mutations of a reference sequence.

    Parameters
    ----------
    args : Namespace
        Command line arguments stored as attributes of a Namespace, usually
        parsed from sys.argv, but can be set programmatically for unit testing
        or other purposes.

    See Also
    --------
    parse_arguments()
    """
    # Input file arg
    in_file_path = args.input_fasta_file
    in_file_name = os.path.basename(in_file_path)
    base_file_name, in_file_ext = os.path.splitext(in_file_name)

    # Random seed option
    random.seed(args.random_seed)

    # Read the reference and generate mutations
    seq_name, seq_str = read_fasta_sequence(in_file_path)

    # Find the eligible positions
    eligible_positions = get_eligible_positions(seq_str)
    eligible_seq_length = len(eligible_positions)

    num_mutations = args.num_subs + args.num_insertions + args.num_deletions
    if num_mutations > eligible_seq_length:
        print("ERROR: You have specified a number of substitutions that is greater than the eligible length of the sequence", file=sys.stderr)
        sys.exit()

    run_simulations(seq_str, eligible_positions, base_file_name, seq_name, args.num_sims, args.num_subs,
                    args.num_insertions, args.num_deletions, args.summary_file)


if __name__ == '__main__':
    args = parse_arguments(sys.argv[1:])
    main(args)





#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_snpmutator
----------------------------------

Tests for `snpmutator` module.
"""

import os
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
import unittest
from testfixtures import TempDirectory
import random
import argparse
from itertools import izip_longest
from snpmutator import snpmutator



def make_random_dna_string(length, allowed_bases):
    """
    Create random DNA sequence string of the specified length.
    
    Parameters
    ----------
    length : int
        The length of the DNA string being generated.
    allowed_bases : str
        A string of possible bases.
        
    Returns
    -------
    str
        Random DNA string
    """
    char_list = [random.choice(allowed_bases) for _ in range(length)]
    return "".join(char_list)
    

def write_fasta(seq_string, directory, file_name, ident="", name="", description=""):
    """
    Write a sequence to a fasta file.
    
    Parameters
    ----------
    seq_string : str
        Sequence of nucleotide characters to write to the fasta file
    directory : str
        Directory of the fasta file to write
    file_name : str
        File name of the fasta file to write
    ident : str, optional
        Identifier such as a locus tag (string)
    name : str, optional
        Sequence name
    description : str, optional
        Additional text
        
    Returns
    -------
    str
        Path of fasta file
    """
    seq = Seq(seq_string)
    record = SeqRecord(seq, ident, name, description)
    file_path = os.path.join(directory, file_name)
    SeqIO.write([record], file_path, "fasta")
    return file_path

    
def write_fixed_dna_fasta(dna, directory, file_name):
    """
    Write a dna string to a fasta file.
    The fasta file will contain a random ID, name, and description.

    Parameters
    ----------
    dna : str
        String of DNA to write to fasta file.
    directory : str
        Directory of the fasta file to write
    file_name : str
        File name of the fasta file to write
        
    Returns
    -------
    file_path : str
        Path of fasta file
    """
    random_num_str = str(random.choice(range(10, 100))) # two digit number
    ident = "Id" + random_num_str
    name = "Name" + random_num_str
    description = "Description" + random_num_str
    file_path = write_fasta(dna, directory, file_name, ident, name, description)
    return file_path


def write_random_dna_fasta(directory, file_name, length, allowed_bases="GATC"):
    """
    Write a random dna string of a specified length to a fasta file.
    The fasta file will also contain a random ID, name, and description.

    Parameters
    ----------
    directory : str
        Directory of the fasta file to write
    file_name : str
        File name of the fasta file to write
    length : int
        Length of nucleotide sequence generate and write to the fasta file
    allowed_bases : str, optional
        A string of possible bases.  Defaults to 'GATC'
        
    Returns
    -------
    file_path : str
        Path of fasta file
    dna : str
        Generated DNA string
    """
    dna = make_random_dna_string(length, allowed_bases)
    file_path = write_fixed_dna_fasta(dna, directory, file_name)
    return (file_path, dna)


def read_fasta_seq_record(file_path):
    """
    Reads a fasta file and returns the first sequence record. Any subsequent sequences are ignored.

    Parameters
    ----------
    file_path : str
        Path to the fasta file
        
    Returns
    -------
    SeqRecord
        Biopython SeqRecord
    """
    with open(file_path) as handle:
        iter = SeqIO.parse(handle, 'fasta')
        return iter.next()


def compare_fasta_files(file_path1, file_path2):
    """
    Determine if two fasta files have equivalent contents.  Not necessarily 
    byte-by-byte identical.

    Parameters
    ----------
    file_path1 : str
        Path to the first file.
    file_path2 : str
        Path to the second file.
        
    Returns
    -------
    bool
        True if the two fasta files are equivalent, False otherwise.
    """
    with open(file_path1) as handle1:
        with open(file_path2) as handle2:
            iter1 = SeqIO.parse(handle1, 'fasta')
            iter2 = SeqIO.parse(handle2, 'fasta')
            fillvalue = object()
            paired_records = izip_longest(iter1, iter2, fillvalue=fillvalue)
            return all(r1.__dict__ == r2.__dict__ for r1, r2 in paired_records)


def compare_mutated_fasta_files(original_file_path, mutated_file_path):
    """
    Determine if two fasta files have equivalent contents after running a 
    zero count mutation on the original file.  The files will not be identical.  
    The fasta defline will differ in the description suffix and the sequence 
    lines may have different lengths.

    Parameters
    ----------
    original_file_path : str
        Path to the original fasta file.
    mutated_file_path : str
        Path to the mutated fasta file.
        
    Returns
    -------
    bool
        True if the two fasta files are equivalent, False otherwise.
    """
    with open(original_file_path) as handle1:
        with open(mutated_file_path) as handle2:
            iter1 = SeqIO.parse(handle1, 'fasta')
            iter2 = SeqIO.parse(handle2, 'fasta')
            fillvalue = object()
            paired_records = izip_longest(iter1, iter2, fillvalue=fillvalue)
            for r1, r2 in paired_records:
                if type(r1) != type(r2):
                    return False
                if r1.id != r2.id:
                    return False
                if r1.name != r2.name:
                    return False
                if r1.seq != r2.seq:
                    return False
                if not r2.description.startswith(r1.description):
                    return False
    return True
                


class TestSnpmutator(unittest.TestCase):

    def setUp(self):
        random.seed(1)

    def test_invalid_args(self):
        """Verify the argument parser rejects invalid arguments
        """
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 100)
        
        sys_arguments = ("-s -1 " + original_file_path).split()
        self.assertRaises(SystemExit, snpmutator.parse_arguments, sys_arguments)

        sys_arguments = ("-i -1 " + original_file_path).split()
        self.assertRaises(SystemExit, snpmutator.parse_arguments, sys_arguments)

        sys_arguments = ("-d -1 " + original_file_path).split()
        self.assertRaises(SystemExit, snpmutator.parse_arguments, sys_arguments)

        sys_arguments = ("-s aa " + original_file_path).split()
        self.assertRaises(SystemExit, snpmutator.parse_arguments, sys_arguments)

        sys_arguments = ("-i aa " + original_file_path).split()
        self.assertRaises(SystemExit, snpmutator.parse_arguments, sys_arguments)

        sys_arguments = ("-d aa " + original_file_path).split()
        self.assertRaises(SystemExit, snpmutator.parse_arguments, sys_arguments)

        sys_arguments = ("-n -1 " + original_file_path).split()
        self.assertRaises(SystemExit, snpmutator.parse_arguments, sys_arguments)

        sys_arguments = ("-n aa " + original_file_path).split()
        self.assertRaises(SystemExit, snpmutator.parse_arguments, sys_arguments)

    def test_zero_changes(self):
        """Verify the output fasta file matches the input fasta file when zero substitions, insertions, and deletions are requested.
        """
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 1000)
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 1
        args.num_subs = 0
        args.num_insertions = 0
        args.num_deletions = 0
        args.random_seed = 1
        args.summary_file = None
        snpmutator.main(args)
        no_change = compare_mutated_fasta_files(original_file_path, "original_mutated_1.fasta")
        self.assertTrue(no_change, "Generated fasta file does not match original fasta file")

    def test_snp_changes(self):
        """Test various numbers of substitutions.
        """
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 10)
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 1
        args.num_insertions = 0
        args.num_deletions = 0
        args.random_seed = 1
        args.summary_file = None

        args.num_subs = 1
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "GCTAAATCGG", "SNP 1 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

        args.num_subs = 5
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "ACTATAGCGC", "SNP 5 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

        args.num_subs = 10
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "AGTCTCGTAC", "SNP 10 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

    def test_eligible_snp_changes(self):
        """Test substitutions where some positions are ineligible
        """
        directory = TempDirectory()
        dna = "12345aaaaaAAAAA12345"
        original_file_path = write_fixed_dna_fasta(dna, directory.path, "original.fasta")
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 1
        args.num_insertions = 0
        args.num_deletions = 0
        args.random_seed = 1
        args.summary_file = None

        args.num_subs = 10
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "12345GGTCTCGTGC12345", "Eligible SNP test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

    def test_insert_changes(self):
        """Test various numbers of insertions.
        """
        
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 10)
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 1
        args.num_subs = 0
        args.num_deletions = 0
        args.random_seed = 1
        args.summary_file = None

        args.num_insertions = 1
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "GCGCAAATCGG", "Insert 1 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

        args.num_insertions = 5
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "CGCGCATAAATCGCG", "Insert 5 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

        args.num_insertions = 10
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "CGACGCTATATAATTCCGCG", "Insert 10 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

    def test_delete_changes(self):
        """Test various numbers of deletions.
        """
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 10)
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 1
        args.num_subs = 0
        args.num_insertions = 0
        args.random_seed = 1
        args.summary_file = None

        args.num_deletions = 1
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "GCAAATCGG", "Delete 1 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

        args.num_deletions = 5
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "CAACG", "Delete 5 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

        args.num_deletions = 10
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "", "Delete 10 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

    def test_mutate_mix_changes(self):
        """Test a mix of substitutions, inserts, and deletes.
        """
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 10)
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 1
        args.random_seed = 1
        args.summary_file = None

        args.num_subs = 1
        args.num_insertions = 1
        args.num_deletions = 1
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "GCTAAAATCG", "Mutate mix 1,1,1 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

        args.num_subs = 2
        args.num_insertions = 2
        args.num_deletions = 2
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "TGCTCAACGC", "Mutate mix 2,2,2 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))

        args.num_subs = 3
        args.num_insertions = 4
        args.num_deletions = 3
        snpmutator.main(args)
        mutated_seq_record = read_fasta_seq_record("original_mutated_1.fasta")
        self.assertEqual(str(mutated_seq_record.seq), "CCTTAGTCAGC", "Mutate mix 3,4,3 test failed, dna=%s mutated seq=%s" % (dna, str(mutated_seq_record.seq)))
            
    def test_too_many_mutations(self):
        """Deliberately exceed the maximum allowed number of mutations.
        """
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 10)
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 1
        args.random_seed = 1
        args.summary_file = None

        args.num_subs = 4
        args.num_insertions = 4
        args.num_deletions = 3
        # Verify exit if number of mutations exceeds sequence length
        self.assertRaises(SystemExit, snpmutator.main, args)
            
    def test_too_many_mutations_ineligible(self):
        """Deliberately exceed the maximum allowed number of mutations when
        some of the reference positions are not ATGC.
        """
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 10, "ACGTSNWR")
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 1
        args.random_seed = 1
        args.summary_file = None

        args.num_subs = 10
        args.num_insertions =0
        args.num_deletions = 0
        # Verify exit if number of mutations exceeds eligible sequence length
        self.assertRaises(SystemExit, snpmutator.main, args)
            
            
    def test_not_all_same(self):
        """Verify Mutator creates different mutated fasta files when generating more than one.
        """
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 1000)
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 3
        args.num_subs = 2
        args.num_insertions = 2
        args.num_deletions = 2
        args.random_seed = 1
        args.summary_file = None
        snpmutator.main(args)
        mutated_seq_record1 = read_fasta_seq_record("original_mutated_1.fasta")
        mutated_seq_record2 = read_fasta_seq_record("original_mutated_2.fasta")
        mutated_seq_record3 = read_fasta_seq_record("original_mutated_3.fasta")
        self.assertNotEqual(str(mutated_seq_record1.seq), str(mutated_seq_record2.seq), "Generated sequences 1 and 2 should be different.")
        self.assertNotEqual(str(mutated_seq_record2.seq), str(mutated_seq_record3.seq), "Generated sequences 2 and 3 should be different.")
        self.assertNotEqual(str(mutated_seq_record1.seq), str(mutated_seq_record3.seq), "Generated sequences 1 and 3 should be different.")

    def test_summary_creation(self):
        """Verify the summary file is created if and only if requested.
        """
        directory = TempDirectory()
        original_file_path, dna = write_random_dna_fasta(directory.path, "original.fasta", 1000)
        args = argparse.Namespace()
        args.input_fasta_file = original_file_path
        args.num_sims = 1
        args.num_subs = 0
        args.num_insertions = 0
        args.num_deletions = 0
        args.random_seed = 1

        args.summary_file = None
        snpmutator.main(args)
        summary_file_exists = os.path.exists("original_snpListMutated.txt")
        self.assertFalse(summary_file_exists, "The summary file should not exist when not explicitly requested")

        args.summary_file = "original_snpListMutated.txt"
        snpmutator.main(args)
        summary_file_exists = os.path.exists("original_snpListMutated.txt")
        self.assertTrue(summary_file_exists, "The summary file is missing when requested.")


    def tearDown(self):
        """
        Delete all the temporary directories and files created during this 
        testing session.
        """
        if os.path.exists("original_mutated_1.fasta"):
            os.remove("original_mutated_1.fasta")
        if os.path.exists("original_mutated_2.fasta"):
            os.remove("original_mutated_2.fasta")
        if os.path.exists("original_mutated_3.fasta"):
            os.remove("original_mutated_3.fasta")
        if os.path.exists("original_snpListMutated.txt"):
            os.remove("original_snpListMutated.txt")
        TempDirectory.cleanup_all()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    'numpy',
    'Biopython',
]

# Below needed for Python 2.6
if sys.version_info < (2,7,):
    requirements.append('argparse')

test_requirements = [
    'testfixtures',
]

setup(
    name='snp-mutator',
    version='0.2.0',
    description="Generate mutated sequence files from a reference genome.",
    long_description=readme + '\n\n' + history,
    author="Hugh Rand",
    author_email='Hugh.Rand@fda.hhs.gov',
    url='https://github.com/CFSAN-Biostatistics/snp-mutator',
    packages=[
        'snpmutator',
    ],
    package_dir={'snpmutator':
                 'snpmutator'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords=['bioinformatics', 'NGS', 'snp-mutator'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    scripts=[
        'snpmutator/snpmutator.py',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
